#ifndef GOAPI_DEF_H
#define GOAPI_DEF_H

#include <kApi/kApiDef.h>

// For std::stoi(), std::to_string() availability on TI compiler.
#include "../extern/lmi-ext/include/lmi-ext/lmi-ext.hpp"

// TODO: hack until kApi supports exporting C++ functions [kExportCppFx => kExport, kImportCppFx => kImport] (GOS-7312).
#if defined(K_MSVC)
#   define kExportCppFx(TYPE)                  __declspec(dllexport) TYPE                           //< Decoration macro to export function with name mangling.
#   define kImportCppFx(TYPE)                  __declspec(dllimport) TYPE                           //< Decoration macro to import function with name mangling (could be no-op, but dllimport allows to generate better code).
#elif defined(K_TI_CCS)
#   define kExportCppFx(TYPE)                  TYPE
#   define kImportCppFx(TYPE)                  TYPE
#else
#   define kExportCppFx(TYPE)                  __attribute__((visibility ("default"))) TYPE         //< Decoration macro to export function with name mangling.
#   define kImportCppFx(TYPE)                  TYPE                                                 //< Decoration macro to import function with name mangling (no-op).
#endif

// GoApiFx was added (renamed old one to GoApiCppFx that was for C++ functions)
// due to need of having PointerWrapper to handle C++ pointers as kApi objects.
// GoApi now handles both C and C++ classes because of this change and
// new projects that include GoApi might need to add dependency to the assembly
// if they are using @ref MsgQueue or @ref PointerWrapper.
#if defined (GOAPI_EXPORT)
#   define GoApiCppFx(TYPE)     kExportCppFx(TYPE)
#   define GoApiFx(TYPE)        kExportFx(TYPE)
#   define GoApiCx(TYPE)        kExportCx(TYPE)
#   define GoApiDx(TYPE)        kExportDx(TYPE)
#   define GoApiClass           kExportClass
#else
#   define GoApiCppFx(TYPE)     kImportCppFx(TYPE)
#   define GoApiFx(TYPE)        kImportFx(TYPE)
#   define GoApiCx(TYPE)        kImportCx(TYPE)
#   define GoApiDx(TYPE)        kImportDx(TYPE)
#   define GoApiClass           kImportClass
#endif

#if defined (K_MSVC)
    //This disables warnings for templated std containers (std::vector, etc) not being dll exported.
    //Note: If you have a templated std container member variable used outside its owner class where the std library isnt available
    //      you will have *serious issues*. This shouldn't be a problem for us, as the GoApi libraries are
    //      internal use only, but still, be aware. Google "Visual Studio Warning C4251" for more info.
    #pragma warning( disable : 4251 )
    #pragma warning( disable : 4275 )
#endif

// Define the formats for text to indicate if the text is associated with a
// context or not, and if the text is translated or not.
// The defines are not used by the executable code, but instead is used to mark
// the text that a parsing script extracts and organizes
// for post-processing, such as sending the text out of translation.
// - _(): belongs to global dictionary context and not translated.
// - pgettext(C, T): belongs to context C and is translated.
// The parameters are:
// - "C" is the context.
// - "T" is the log text.
//
// Text translation is needed for:
// - user visible logs (most obvious use)
// - read-only and initial display names, parameter titles etc.
// Therefore these definitions are not defined in the GcLogger, but instead
// is defined here so they be used in non-logging cases, such as tool names etc.
//
// NOTE: Vision Engineering (VE) code repo also uses these macro names to identify
// translatable text. So if there is a change to these names in the Gocator
// code repo, then ensure the VE repo makes the same changes otherwise the
// scripts to extract text tagged with these macros could miss the VE repo's
// tagged text.
// TODO: remove the #ifndef once the same definitions in GvDef.h is removed.
#ifndef pgettext
#define pgettext(C, T)          T
#endif
#ifndef _
#define _(T)                    T
#endif

namespace Go
{
    namespace Properties
    {
        class Structure;

        namespace Internal
        {
            class ArrayBase;
            class ReferenceArray;
            class ValueBase;
        }
    }
}

#endif
