#include "MemoryLeakDetection.h"

#if defined(K_WINDOWS)
#include <windows.h>    // For DebugApi.h::IsDebuggerPresent()
#endif

namespace Go {


MemoryLeakDetection::MemoryLeakDetection(): serviceMemState()
{
}

MemoryLeakDetection::~MemoryLeakDetection()
{
}

#if defined(_DEBUG) && defined(_MSC_VER)

void MemoryLeakDetection::CreateSystemLeakCheckpoint()
{
    _CrtMemCheckpoint(&serviceMemState.memCheckpoint);
}

bool MemoryLeakDetection::HasSystemLeaks() const
{
    GoMemState now, diff;

    _CrtMemCheckpoint(&now);

    return _CrtMemDifference(&diff, &serviceMemState.memCheckpoint, &now);
}

void MemoryLeakDetection::PrintSystemLeaks() const
{
    int reportMode = _CRTDBG_MODE_FILE;

#if defined(K_WINDOWS)
    if (IsDebuggerPresent())
    {
        // this also outputs to the Visual Studio Debugger Output window if attached.
        reportMode |= _CRTDBG_MODE_DEBUG;
    }
#endif

    _CrtSetReportMode(_CRT_WARN, reportMode);
    _CrtSetReportFile(_CRT_WARN, _CRTDBG_FILE_STDOUT);
    _CrtSetReportMode(_CRT_ERROR, reportMode);
    _CrtSetReportFile(_CRT_ERROR, _CRTDBG_FILE_STDOUT);
    _CrtSetReportMode(_CRT_ASSERT, reportMode);
    _CrtSetReportFile(_CRT_ASSERT, _CRTDBG_FILE_STDOUT);

    _CrtMemDumpAllObjectsSince(&serviceMemState.memCheckpoint);
}

#else

void MemoryLeakDetection::CreateSystemLeakCheckpoint()
{
}

bool MemoryLeakDetection::HasSystemLeaks() const
{
    return false;
}

void MemoryLeakDetection::PrintSystemLeaks() const
{
}

#endif

} // namespace
