/**
 * @file    GoGdpFeatureCircle.cs
 *
 * @internal
 * Copyright (C) 2022-2023 by LMI Technologies Inc.
 * Licensed under the MIT License.
 * Redistributed files must retain the above copyright notice.
 */
namespace GoPxLSdk.GoGdpMsg
{
    public class GoGdpFeatureCircle : GoGdpMsg
    {
        /// <summary>Get x coordinate of the circle center point.</summary>
        /// <returns>x coordinate value of the circle center point.</returns>
        public double CenterX { get; private set; }

        /// <summary>Get y coordinate of the circle center point.</summary>
        /// <returns>y coordinate value of the circle center point.</returns>
        public double CenterY { get; private set; }

        /// <summary>Get z coordinate of the circle center point.</summary>
        /// <returns>z coordinate value of the circle center point.</returns>
        public double CenterZ { get; private set; }

        /// <summary>Get x component of the normal vector.</summary>
        /// <returns>x component of normal vector.</returns>
        public double NormalX { get; private set; }

        /// <summary>Get y component of the normal vector.</summary>
        /// <returns>y component of normal vector.</returns>
        public double NormalY { get; private set; }

        /// <summary>Get z component of the normal vector.</summary>
        /// <returns>z component of normal vector.</returns>
        public double NormalZ { get; private set; }

        /// <summary>Get radius of the circle.</summary>
        /// <returns>radius value of the circle.</returns>
        public double Radius { get; private set; }

        /// <summary>Constructs class GoGdpFeatureCircle.</summary>
        public GoGdpFeatureCircle()
            : base(MessageType.CircleFeature)
        { }

        internal GoGdpFeatureCircle(GoPoint3d64f centre, GoPoint3d64f normal, double radius)
            : base(MessageType.CircleFeature)
        {
            CenterX = centre.X;
            CenterY = centre.Y;
            CenterZ = centre.Z;

            NormalX = normal.X;
            NormalY = normal.Y;
            NormalZ = normal.Z;

            Radius = radius;
        }

        /// <summary>Deserialize gdp circle feature message.</summary>
        /// <param name="serializer">The serializer to read.</param>
        /// <exception cref="ArgumentOutOfRangeException">Thrown if failed to deserialize feature circle message.</exception>
        internal override void Deserialize(GoBinaryReader serializer)
        {
            base.Deserialize(serializer);

            CenterX = serializer.ReadDouble();
            CenterY = serializer.ReadDouble();
            CenterZ = serializer.ReadDouble();
            NormalX = serializer.ReadDouble();
            NormalY = serializer.ReadDouble();
            NormalZ = serializer.ReadDouble();
            Radius = serializer.ReadDouble();
        }
    }
}