/**
 * @file    GoGdpFeaturePlane.cs
 *
 * @internal
 * Copyright (C) 2022-2023 by LMI Technologies Inc.
 * Licensed under the MIT License.
 * Redistributed files must retain the above copyright notice.
 */
namespace GoPxLSdk.GoGdpMsg
{
    public class GoGdpFeaturePlane : GoGdpMsg
    {
        /// <summary>Get x component of the normal vector.</summary>
        /// <returns>x component of normal vector.</returns>
        public double NormalX { get; private set; }

        /// <summary>Get y component of the normal vector.</summary>
        /// <returns>y component of normal vector.</returns>
        public double NormalY { get; private set; }

        /// <summary>Get z component of the normal vector.</summary>
        /// <returns>z component of normal vector.</returns>
        public double NormalZ { get; private set; }

        /// <summary>Get distance to origin.</summary>
        /// <returns>distance value to origin..</returns>
        public double DistanceToOrigin { get; private set; }

        /// <summary>Constructs class GoGdpFeaturePlane.</summary>
        public GoGdpFeaturePlane()
            : base(MessageType.PlaneFeature) { }

        internal GoGdpFeaturePlane(GoPoint3d64f normal, double distanceToOriginx)
            : base(MessageType.PlaneFeature)
        {
            NormalX = normal.X;
            NormalY = normal.Y;
            NormalZ = normal.Z;

            DistanceToOrigin = distanceToOriginx;
        }

        /// <summary>Deserialize gdp plane feature message.</summary>
        /// <param name="serializer">The serializer to read.</param>
        /// <exception cref="ArgumentOutOfRangeException">Thrown if failed to deserialize feature plane message.</exception>
        internal override void Deserialize(GoBinaryReader serializer)
        {
            base.Deserialize(serializer);

            NormalX = serializer.ReadDouble();
            NormalY = serializer.ReadDouble();
            NormalZ = serializer.ReadDouble();

            DistanceToOrigin = serializer.ReadDouble();
        }
    }
}