/**
 * @file    GoGdpFeaturePoint.cs
 *
 * @internal
 * Copyright (C) 2022-2023 by LMI Technologies Inc.
 * Licensed under the MIT License.
 * Redistributed files must retain the above copyright notice.
 */
namespace GoPxLSdk.GoGdpMsg
{
    public class GoGdpFeaturePoint : GoGdpMsg
    {
        /// <summary>Get x coordinate of the position.</summary>
        /// <returns>x coordinate value of the position.</returns>
        public double PositionX { get; private set; }

        /// <summary>Get y coordinate of the position.</summary>
        /// <returns>y coordinate value of the position.</returns>
        public double PositionY { get; private set; }

        /// <summary>Get z coordinate of the position.</summary>
        /// <returns>z coordinate value of the position.</returns>
        public double PositionZ { get; private set; }

        /// <summary>Constructs class GoGdpFeaturePoint.</summary>
        public GoGdpFeaturePoint()
            : base(MessageType.PointFeature)
        { }

        internal GoGdpFeaturePoint(GoPoint3d64f point)
            : base(MessageType.PointFeature)
        {
            PositionX = point.X;
            PositionY = point.Y;
            PositionZ = point.Z;
        }

        /// <summary>Deserialize gdp point feature message.</summary>
        /// <param name="serializer">The serializer to read.</param>
        /// <exception cref="ArgumentOutOfRangeException">if failed to deserialize feature point message.</exception>
        internal override void Deserialize(GoBinaryReader serializer)
        {
            base.Deserialize(serializer);

            PositionX = serializer.ReadDouble();
            PositionY = serializer.ReadDouble();
            PositionZ = serializer.ReadDouble();
        }
    }
}