/**
 * @file    GoGdpMeasurement.cs
 *
 * @internal
 * Copyright (C) 2022-2023 by LMI Technologies Inc.
 * Licensed under the MIT License.
 * Redistributed files must retain the above copyright notice.
 */
namespace GoPxLSdk.GoGdpMsg
{
    public class GoGdpMeasurement : GoGdpMsg
    {
        /// <summary>Get measurement value.</summary>
        public double Value { get; private set; }

        /// <summary>Get measurement decision.</summary>
        public byte Decision { get; private set; }

        /// <summary>Deserialize measurement message.</summary>
        /// <param name="serializer">The serializer to read.</param>
        /// <exception cref="ArgumentOutOfRangeException">Thrown if failed to deserialize measurement message.</exception>
        internal override void Deserialize(GoBinaryReader serializer)
        {
            base.Deserialize(serializer);

            Value = serializer.ReadDouble();
            Decision = serializer.ReadByte();
        }

        /// <summary>Constructs GoGdpMeasurement.</summary>
        public GoGdpMeasurement()
            : base(MessageType.Measurement)
        { }

        internal GoGdpMeasurement(double value, byte decision)
            : base(MessageType.Measurement)
        {
            Value = value;
            Decision = decision;
        }
    }
}