/**
 * @file    GoGdpStamp.cs
 *
 * @internal
 * Copyright (C) 2022-2023 by LMI Technologies Inc.
 * Licensed under the MIT License.
 * Redistributed files must retain the above copyright notice.
 */
namespace GoPxLSdk.GoGdpMsg
{
    public class GoGdpStamp : GoGdpMsg
    {
        /// <summary>Get the stamp frame index.</summary>
        public ulong FrameIndex { get; private set; }

        /// <summary>Get the stamp time stamp.</summary>
        public ulong Timestamp { get; private set; }

        /// <summary>Get the stamp encoder.</summary>
        public long Encoder { get; private set; }

        /// <summary>Get the stamp encoder at Z.</summary>
        public long EncoderAtZ { get; private set; }

        /// <summary>Get the stamp status.</summary>
        public ulong Status { get; private set; }

        /// <summary>Get the stamp PTP time.</summary>
        public ulong PtpTime { get; private set; }

        /// <summary>Constructs GoGdpStamp.</summary>
        public GoGdpStamp()
            : base(MessageType.Stamp)
        { }

        internal GoGdpStamp(ulong frameIndex, ulong timestamp, long encoder, long encoderAtZ, ulong status, ulong ptpTime)
            : base(MessageType.Stamp)
        {
            FrameIndex = frameIndex;
            Timestamp = timestamp;
            Encoder = encoder;
            EncoderAtZ = encoderAtZ;
            Status = status;
            PtpTime = ptpTime;
        }

        /// <summary>Deserialize stamp message.</summary>
        /// <param name="serializer">The serializer to read.</param>
        /// <exception cref="ArgumentOutOfRangeException">Thrown if failed to deserialize stamp message.</exception>
        internal override void Deserialize(GoBinaryReader serializer)
        {
            base.Deserialize(serializer);
            serializer.BeginRead<ushort>(true);

            try
            {
                FrameIndex = serializer.ReadUInt64();
                Timestamp = serializer.ReadUInt64();
                Encoder = serializer.ReadInt64();
                EncoderAtZ = serializer.ReadInt64();
                Status = serializer.ReadUInt64();
                PtpTime = serializer.ReadUInt64();
            }
            finally
            {
                serializer.EndRead();
            }
        }
    }
}