/**
 * @file    GoPoint3d32f.cs
 *
 * @internal
 * Copyright (C) 2022-2023 by LMI Technologies Inc.
 * Licensed under the MIT License.
 * Redistributed files must retain the above copyright notice.
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace GoPxLSdk
{
    public class GoPoint3d32f
    {
        /// <summary>
        /// X-coordinate value.
        /// </summary>
        public float X { get; }

        /// <summary>
        /// Y-coordinate value.
        /// </summary>
        public float Y { get; }

        /// <summary>
        /// Z-coordinate value.
        /// </summary>
        public float Z { get; }

        public GoPoint3d32f(float x, float y, float z)
        {
            X = x;
            Y = y;
            Z = z;
        }

        public static bool operator ==(GoPoint3d32f a, GoPoint3d32f b)
        {
            return a.Equals(b);
        }

        public static bool operator !=(GoPoint3d32f a, GoPoint3d32f b)
        {
            return (!a.Equals(b));
        }

        public sealed override string ToString()
        {
            return $"{X}, {Y}, {Z}";
        }

        public override bool Equals(object obj)
        {
            return obj is GoPoint3d32f f &&
                   X == f.X &&
                   Y == f.Y &&
                   Z == f.Z;
        }

        public override int GetHashCode()
        {
            int hashCode = -307843816;
            hashCode = hashCode * -1521134295 + X.GetHashCode();
            hashCode = hashCode * -1521134295 + Y.GetHashCode();
            hashCode = hashCode * -1521134295 + Z.GetHashCode();
            return hashCode;
        }
    }
}
