/**
 * @file    Def.cs
 * @brief   Declares internal definitions.
 *
 * @internal
 * Copyright (C) 2022-2023 by LMI Technologies Inc.
 * Licensed under the MIT License.
 * Redistributed files must retain the above copyright notice.
 */
using System;
using System.Collections.Generic;

namespace GoPxLSdk
{
    internal static class Def
    {
        // GOS-5782: GoPxl Ports changed from Gocator. These ports sync with the defines in GoCommon/Network/NetworkPorts.h.
        internal const int GO_PXL_SDK_DEFAULT_CONTROL_PORT = 3600;
        internal const int GO_PXL_SDK_DEFAULT_GDP_SERVER_PORT = 3601;
        internal const int GO_PXL_SDK_DEFAULT_WEB_PORT = 8100;

        internal const int DEFAULT_TCP_TIMEOUT_MILLISECONDS = 1000000;

        /// <summary>
        /// Gives the NULL value for primitive types as defined by kApi.
        /// </summary>
        internal static T NullType<T>()
        {
            if (NullTypes.TryGetValue(typeof(T), out object value))
            {
                return (T)value;
            }

            throw new KeyNotFoundException();
        }

        /// <summary>
        /// Gives the NULL value for primitive types as defined by kApi.
        /// </summary>
        ///
        /// <remarks>
        /// KApi dependency was removed from this project, but the data we receive may still include these values. Ideally we are agnostic to these values, but in case we need to compare to, or write, these values they are included here. Signed values are use Min value, Unsigned values use Max.
        /// </remarks>
        private static readonly Dictionary<Type, object> NullTypes = new Dictionary<Type, object>()
        {
            { typeof(float), float.MinValue },
            { typeof(double), double.MinValue },
            { typeof(sbyte), sbyte.MinValue},
            { typeof(short), short.MinValue },
            { typeof(int), int.MinValue},
            { typeof(long), long.MinValue },
            { typeof(byte), byte.MaxValue },
            { typeof(ushort), ushort.MaxValue },
            { typeof(uint), uint.MaxValue },
            { typeof(ulong), ulong.MaxValue},
        };
    }
}
